@@ -2,11 +2,11 @@
 #include "mpeg3private.h"
 #include "mpeg3protos.h"
 
+#include <errno.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
 
-#define MAX(a, b) ((a) > (b) ? (a) : (b))
 
 int mpeg3_major()
 {
@@ -31,25 +31,47 @@ mpeg3_t* mpeg3_new(char *path)
 	mpeg3_t *file = calloc(1, sizeof(mpeg3_t));
 	file->cpus = 1;
 	file->fs = mpeg3_new_fs(path);
-	file->have_mmx = mpeg3_mmx_test();
+// Late compilers don't produce usable code.
 	file->demuxer = mpeg3_new_demuxer(file, 0, 0, -1);
 	file->seekable = 1;
+	file->index_bytes = 0x300000;
 	return file;
 }
 
+mpeg3_index_t* mpeg3_new_index()
+{
+	mpeg3_index_t *index = calloc(1, sizeof(mpeg3_index_t));
+	index->index_zoom = 1;
+	return index;
+}
+
+void mpeg3_delete_index(mpeg3_index_t *index)
+{
+	int i;
+	for(i = 0;i < index->index_channels; i++)
+		free(index->index_data[i]);
+	free(index->index_data);
+	free(index);
+}
+
 int mpeg3_delete(mpeg3_t *file)
 {
 	int i;
+	const int debug = 0;
 
+if(debug) printf("mpeg3_delete 1\n");
 	for(i = 0; i < file->total_vstreams; i++)
 		mpeg3_delete_vtrack(file, file->vtrack[i]);
+if(debug) printf("mpeg3_delete 1\n");
 
 	for(i = 0; i < file->total_astreams; i++)
 		mpeg3_delete_atrack(file, file->atrack[i]);
 
+if(debug) printf("mpeg3_delete 1\n");
 	mpeg3_delete_fs(file->fs);
 	mpeg3_delete_demuxer(file->demuxer);
 
+if(debug) printf("mpeg3_delete 1\n");
 	if(file->frame_offsets)
 	{
 		for(i = 0; i < file->total_vstreams; i++)
@@ -58,12 +80,14 @@ int mpeg3_delete(mpeg3_t *file)
 			free(file->keyframe_numbers[i]);
 		}
 
+if(debug) printf("mpeg3_delete 1\n");
 		free(file->frame_offsets);
 		free(file->keyframe_numbers);
 		free(file->total_frame_offsets);
 		free(file->total_keyframe_numbers);
 	}
 
+if(debug) printf("mpeg3_delete 1\n");
 	if(file->sample_offsets)
 	{
 		for(i = 0; i < file->total_astreams; i++)
@@ -72,10 +96,25 @@ int mpeg3_delete(mpeg3_t *file)
 		free(file->sample_offsets);
 		free(file->total_sample_offsets);
 	}
+
+if(debug) printf("mpeg3_delete 1\n");
+
 	if(file->channel_counts)
 		free(file->channel_counts);
 
+if(debug) printf("mpeg3_delete 2\n");
+	if(file->indexes)
+	{
+		for(i = 0; i < file->total_indexes; i++)
+			mpeg3_delete_index(file->indexes[i]);
+if(debug) printf("mpeg3_delete 3\n");
+			
+		free(file->indexes);
+	}
+
+if(debug) printf("mpeg3_delete 4\n");
 	free(file);
+if(debug) printf("mpeg3_delete 5\n");
 	return 0;
 }
 
@@ -137,373 +176,156 @@ int mpeg3_check_sig(char *path)
 }
 
 
-static uint32_t read_int32(unsigned char *buffer, int *position)
-{
-	uint32_t temp;
 
-	if(MPEG3_LITTLE_ENDIAN)
-	{
-		((unsigned char*)&temp)[3] = buffer[(*position)++];
-		((unsigned char*)&temp)[2] = buffer[(*position)++];
-		((unsigned char*)&temp)[1] = buffer[(*position)++];
-		((unsigned char*)&temp)[0] = buffer[(*position)++];
-	}
+
+
+
+
+
+
+
+static int is_toc(uint32_t bits)
+{
+	return (bits == MPEG3_TOC_PREFIX);
+}
+
+
+static int is_ifo(uint32_t bits)
+{
+	return (bits == MPEG3_IFO_PREFIX);
+}
+
+static int is_transport(uint32_t bits)
+{
+	return (((bits >> 24) & 0xff) == MPEG3_SYNC_BYTE);
+}
+
+static int is_program(uint32_t bits)
+{
+	return (bits == MPEG3_PACK_START_CODE);
+}
+
+static int is_mpeg_audio(uint32_t bits)
+{
+	return (bits & 0xfff00000) == 0xfff00000 ||
+		(bits & 0xffff0000) == 0xffe30000 ||
+		((bits >> 8) == MPEG3_ID3_PREFIX) ||
+		(bits == MPEG3_RIFF_CODE);
+}
+
+static int is_mpeg_video(uint32_t bits)
+{
+	return (bits == MPEG3_SEQUENCE_START_CODE ||
+			bits == MPEG3_PICTURE_START_CODE);
+}
+
+static int is_ac3(uint32_t bits)
+{
+	return (((bits & 0xffff0000) >> 16) == MPEG3_AC3_START_CODE);
+}
+
+static int calculate_packet_size(int is_transport,
+	int is_program,
+	int is_audio,
+	int is_video)
+{
+	if(is_transport)
+		return MPEG3_TS_PACKET_SIZE;
 	else
-	{
-		((unsigned char*)&temp)[0] = buffer[(*position)++];
-		((unsigned char*)&temp)[1] = buffer[(*position)++];
-		((unsigned char*)&temp)[2] = buffer[(*position)++];
-		((unsigned char*)&temp)[3] = buffer[(*position)++];
-	}
-	
-	return temp;
-}
-
-static uint64_t read_int64(unsigned char *buffer, int *position)
-{
-	uint64_t temp;
-
-	if(MPEG3_LITTLE_ENDIAN)
-	{
-		((unsigned char*)&temp)[7] = buffer[(*position)++];
-		((unsigned char*)&temp)[6] = buffer[(*position)++];
-		((unsigned char*)&temp)[5] = buffer[(*position)++];
-		((unsigned char*)&temp)[4] = buffer[(*position)++];
-		((unsigned char*)&temp)[3] = buffer[(*position)++];
-		((unsigned char*)&temp)[2] = buffer[(*position)++];
-		((unsigned char*)&temp)[1] = buffer[(*position)++];
-		((unsigned char*)&temp)[0] = buffer[(*position)++];
-	}
-	else
-	{
-		((unsigned char*)&temp)[0] = buffer[(*position)++];
-		((unsigned char*)&temp)[1] = buffer[(*position)++];
-		((unsigned char*)&temp)[2] = buffer[(*position)++];
-		((unsigned char*)&temp)[3] = buffer[(*position)++];
-		((unsigned char*)&temp)[4] = buffer[(*position)++];
-		((unsigned char*)&temp)[5] = buffer[(*position)++];
-		((unsigned char*)&temp)[6] = buffer[(*position)++];
-		((unsigned char*)&temp)[7] = buffer[(*position)++];
-	}
-
-	return temp;
-}
-
-
-
-
-
-
-static int read_toc(mpeg3_t *file)
-{
-	unsigned char *buffer;
-	int file_type;
-	int position = 4;
-	int stream_type;
-	int atracks;
-	int vtracks;
-	int i, j;
-	int is_vfs = 0;
-	if(!strncmp(file->fs->path, RENDERFARM_FS_PREFIX, strlen(RENDERFARM_FS_PREFIX)))
-		is_vfs = 1;
-//printf("read_toc 10\n");
-	buffer = malloc(mpeg3io_total_bytes(file->fs));
-//printf("read_toc 10\n");
-	mpeg3io_seek(file->fs, 0);
-//printf("read_toc 10\n");
-	mpeg3io_read_data(buffer, mpeg3io_total_bytes(file->fs), file->fs);
-//printf("read_toc 10\n");
-
-
-//printf("read_toc %lld\n", mpeg3io_total_bytes(file->fs));
-
-// File type
-	file_type = buffer[position++];
-	switch(file_type)
-	{
-		case FILE_TYPE_PROGRAM:
-			file->is_program_stream = 1;
-			break;
-		case FILE_TYPE_TRANSPORT:
-			file->is_transport_stream = 1;
-			break;
-		case FILE_TYPE_AUDIO:
-			file->is_audio_stream = 1;
-			break;
-		case FILE_TYPE_VIDEO:
-			file->is_video_stream = 1;
-			break;
-	}
-
-//printf("read_toc 10\n");
-
-// Stream ID's
-	while((stream_type = buffer[position]) != TITLE_PATH)
-	{
-		int offset;
-		int stream_id;
-
-//printf("read_toc %d %x\n", position, buffer[position]);
-		position++;
-		offset = read_int32(buffer, &position);
-		stream_id = read_int32(buffer, &position);
-
-		if(stream_type == STREAM_AUDIO)
-		{
-			file->demuxer->astream_table[offset] = stream_id;
-		}
-
-		if(stream_type == STREAM_VIDEO)
-		{
-			file->demuxer->vstream_table[offset] = stream_id;
-		}
-	}
-
-
-
-//printf("read_toc 10\n");
-
-
-// Titles
-	while(buffer[position] == TITLE_PATH)
-	{
-		char string[MPEG3_STRLEN];
-		int string_len = 0;
-		mpeg3_title_t *title;
-
-		position++;
-		if(is_vfs)
-		{
-			strcpy(string, RENDERFARM_FS_PREFIX);
-			string_len = strlen(string);
-		}
-		while(buffer[position] != 0) string[string_len++] = buffer[position++];
-		string[string_len++] = 0;
-		position++;
-
-		title = 
-			file->demuxer->titles[file->demuxer->total_titles++] = 
-			mpeg3_new_title(file, string);
-
-		title->total_bytes = read_int64(buffer, &position);
-
-// Cells
-		title->timecode_table_size = 
-			title->timecode_table_allocation = 
-			read_int32(buffer, &position);
-		title->timecode_table = calloc(title->timecode_table_size, sizeof(mpeg3demux_timecode_t));
-		for(i = 0; i < title->timecode_table_size; i++)
-		{
-			title->timecode_table[i].start_byte = read_int64(buffer, &position);
-			title->timecode_table[i].end_byte = read_int64(buffer, &position);
-			title->timecode_table[i].program = read_int32(buffer, &position);
-		}
-	}
-//printf("read_toc 10\n");
-
-
-
-// Audio streams
-// Skip ATRACK_COUNT
-	position++;
-	atracks = read_int32(buffer, &position);
-//printf("read_toc 10\n");
-
-// Skip VTRACK_COUNT
-	position++;
-	vtracks = read_int32(buffer, &position);
-//printf("read_toc 10\n");
-
-
-	if(atracks)
-	{
-		file->channel_counts = calloc(sizeof(int), atracks);
-		file->sample_offsets = malloc(sizeof(int64_t*) * atracks);
-		file->total_sample_offsets = malloc(sizeof(int*) * atracks);
-
-		for(i = 0; i < atracks; i++)
-		{
-			file->channel_counts[i] = read_int32(buffer, &position);
-			file->total_sample_offsets[i] = read_int32(buffer, &position);
-			file->sample_offsets[i] = malloc(file->total_sample_offsets[i] * sizeof(int64_t));
-			for(j = 0; j < file->total_sample_offsets[i]; j++)
-			{
-				file->sample_offsets[i][j] = read_int64(buffer, &position);
-//printf("samples %llx\n", file->sample_offsets[i][j]);
-			}
-		}
-	}
-//printf("read_toc 10\n");
-
-	if(vtracks)
-	{
-		file->frame_offsets = malloc(sizeof(int64_t*) * vtracks);
-		file->total_frame_offsets = malloc(sizeof(int*) * vtracks);
-		file->keyframe_numbers = malloc(sizeof(int64_t*) * vtracks);
-		file->total_keyframe_numbers = malloc(sizeof(int*) * vtracks);
-
-		for(i = 0; i < vtracks; i++)
-		{
-			file->total_frame_offsets[i] = read_int32(buffer, &position);
-			file->frame_offsets[i] = malloc(file->total_frame_offsets[i] * sizeof(int64_t));
-			for(j = 0; j < file->total_frame_offsets[i]; j++)
-			{
-				file->frame_offsets[i][j] = read_int64(buffer, &position);
-//printf("frame %llx\n", file->frame_offsets[i][j]);
-			}
-
-
-			file->total_keyframe_numbers[i] = read_int32(buffer, &position);
-			file->keyframe_numbers[i] = malloc(file->total_keyframe_numbers[i] * sizeof(int64_t));
-			for(j = 0; j < file->total_keyframe_numbers[i]; j++)
-			{
-				file->keyframe_numbers[i][j] = read_int64(buffer, &position);
-			}
-		}
-	}
-//printf("read_toc 10\n");
-
-	free(buffer);
-//printf("read_toc 10\n");
-
-
-
-//printf("read_toc 1\n");
-	mpeg3demux_open_title(file->demuxer, 0);
-//printf("read_toc 10\n");
-
-//printf("read_toc 2 %llx\n", mpeg3demux_tell(file->demuxer));
-	return 0;
-}
-
-
-
-
-mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
-{
-	mpeg3_t *file = 0;
-	unsigned int bits;
-	int i, done;
-
-/* Initialize the file structure */
-	file = mpeg3_new(path);
-
-
-
-
-
-
-
-//printf("mpeg3_open_copy 1 %s\n", path);
-
-/* Need to perform authentication before reading a single byte. */
-	if(mpeg3io_open_file(file->fs))
-	{
-		mpeg3_delete(file);
+	if(is_program)
 		return 0;
-	}
+	else
+	if(is_audio)
+		return MPEG3_DVD_PACKET_SIZE;
+	else
+	if(is_video)
+		return MPEG3_DVD_PACKET_SIZE;
+}
 
+int mpeg3_get_file_type(mpeg3_t *file, 
+	mpeg3_t *old_file,
+	int *toc_atracks,
+	int *toc_vtracks)
+{
+	uint32_t bits = mpeg3io_read_int32(file->fs);
 
-
-
-
-
-
-
-
-
-
-
-
-/* =============================== Create the title objects ========================= */
-	bits = mpeg3io_read_int32(file->fs);
-//printf("mpeg3_open 1 %p %d %d %d %d\n", old_file, file->is_transport_stream, file->is_program_stream, file->is_video_stream, file->is_audio_stream);
-
-	if(bits == MPEG3_TOC_PREFIX)   /* TOC  */
+/* TOC  */
+	if(is_toc(bits))   
 	{
 /* Table of contents for another title set */
 		if(!old_file)
 		{
-//printf("libmpeg3 10\n");
-			if(read_toc(file))
+			if(toc_atracks && toc_vtracks)
 			{
-//printf("libmpeg3 20\n");
+				if(mpeg3_read_toc(file, toc_atracks, toc_vtracks))
+				{
+					mpeg3io_close_file(file->fs);
+					mpeg3_delete(file);
+					return 1;
+				}
+			}
+			else
+			{
 				mpeg3io_close_file(file->fs);
-//printf("libmpeg3 21n");
 				mpeg3_delete(file);
-//printf("libmpeg3 22\n");
-				return 0;
+				return 1;
 			}
-//printf("libmpeg3 30\n");
 		}
 		mpeg3io_close_file(file->fs);
-//printf("libmpeg3 40\n");
 	}
 	else
-// IFO file
-    if(bits == MPEG3_IFO_PREFIX)
+/* IFO file */
+    if(is_ifo(bits))
     {
-//printf("libmpeg3 1\n");
     	if(!old_file)
 		{
-//printf("libmpeg3 2\n");
+			file->is_program_stream = 1;
 			if(mpeg3_read_ifo(file, 0))
         	{
 				mpeg3_delete(file);
 				mpeg3io_close_file(file->fs);
-				return 0;
+				return 1;
         	}
-//printf("libmpeg3 3\n");
 		}
 		file->is_ifo_file = 1;
 		mpeg3io_close_file(file->fs);
     }
     else
-	if(((bits >> 24) & 0xff) == MPEG3_SYNC_BYTE)
+	if(is_transport(bits))
 	{
 /* Transport stream */
-//printf("libmpeg3 2\n");
 		file->is_transport_stream = 1;
 	}
 	else
-	if(bits == MPEG3_PACK_START_CODE)
+	if(is_program(bits))
 	{
 /* Program stream */
 /* Determine packet size empirically */
-//printf("libmpeg3 3\n");
 		file->is_program_stream = 1;
 	}
 	else
-	if((bits & 0xfff00000) == 0xfff00000 ||
-		(bits & 0xffff0000) == 0xffe30000 ||
-		((bits >> 8) == MPEG3_ID3_PREFIX) ||
-		(bits == MPEG3_RIFF_CODE))
+	if(is_mpeg_audio(bits))
 	{
 /* MPEG Audio only */
-//printf("libmpeg3 4\n");
 		file->is_audio_stream = 1;
 	}
 	else
-	if(bits == MPEG3_SEQUENCE_START_CODE ||
-		bits == MPEG3_PICTURE_START_CODE)
+	if(is_mpeg_video(bits))
 	{
 /* Video only */
-//printf("libmpeg3 5\n");
 		file->is_video_stream = 1;
 	}
 	else
-	if(((bits & 0xffff0000) >> 16) == MPEG3_AC3_START_CODE)
+	if(is_ac3(bits))
 	{
 /* AC3 Audio only */
-//printf("libmpeg3 6\n");
 		file->is_audio_stream = 1;
 	}
 	else
 	{
-//printf("libmpeg3 7\n");
 		mpeg3_delete(file);
-		fprintf(stderr, "mpeg3_open: not an MPEG 2 stream\n");
-		return 0;
+		fprintf(stderr, "mpeg3_get_file_type: not a readable stream.\n");
+		return 1;
 	}
 
 /*
@@ -516,29 +338,45 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
  */
 
 
+	file->packet_size = calculate_packet_size(file->is_transport_stream,
+		file->is_program_stream,
+		file->is_audio_stream,
+		file->is_video_stream);
+
+	return 0;
+}
+
+
+mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
+{
+	mpeg3_t *file = 0;
+	int i, done;
+/* The table of contents may have fewer tracks than are in the demuxer */
+/* This limits the track count */
+	int toc_atracks = 0x7fffffff;
+	int toc_vtracks = 0x7fffffff;
+
+/* Initialize the file structure */
+	file = mpeg3_new(path);
+
+
+
+/* Need to perform authentication before reading a single byte. */
+	if(mpeg3io_open_file(file->fs))
+	{
+		mpeg3_delete(file);
+		return 0;
+	}
 
 
 
 
 
-//printf("mpeg3_open 20\n");
-
-
-
-
-
-// Configure packet size
-	if(file->is_transport_stream)
-		file->packet_size = MPEG3_TS_PACKET_SIZE;
-	else
-	if(file->is_program_stream)
-		file->packet_size = 0;
-	else
-	if(file->is_audio_stream)
-		file->packet_size = MPEG3_DVD_PACKET_SIZE;
-	else
-	if(file->is_video_stream)
-		file->packet_size = MPEG3_DVD_PACKET_SIZE;
+/* =============================== Create the title objects ========================= */
+	if(mpeg3_get_file_type(file, 
+		old_file,
+		&toc_atracks, 
+		&toc_vtracks)) return 0;
 
 
 
@@ -550,10 +388,6 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 
 
 
-
-
-//printf("mpeg3_open 1\n");
-
 /* Create titles */
 /* Copy timecodes from an old demuxer */
 	if(old_file && mpeg3_get_demuxer(old_file))
@@ -566,12 +400,11 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 /* Start from scratch */
 	if(!file->demuxer->total_titles)
 	{
-		mpeg3demux_create_title(file->demuxer, 0, 0);
+		mpeg3_create_title(file->demuxer, 0);
 	}
 
 
 
-//printf("mpeg3_open 50\n");
 
 
 
@@ -587,24 +420,9 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 	if(file->is_transport_stream || file->is_program_stream)
 	{
 /* Create video tracks */
-/* Video must be created before audio because audio uses the video timecode */
-/* to get its length. */
-/*
- * for(i = 0; i < MPEG3_MAX_STREAMS; i++)
- * {
- * 	if(file->demuxer->vstream_table[i])
- * 		printf(__FUNCTION__ " %x %x\n", i, file->demuxer->vstream_table[i]);
- * }
- * for(i = 0; i < MPEG3_MAX_STREAMS; i++)
- * {
- * 	if(file->demuxer->astream_table[i])
- * 		printf(__FUNCTION__ " %x %x\n", i, file->demuxer->astream_table[i]);
- * }
- */
-
-
-//printf("mpeg3_open 3\n");
-		for(i = 0; i < MPEG3_MAX_STREAMS; i++)
+		for(i = 0; 
+			i < MPEG3_MAX_STREAMS && file->total_vstreams < toc_vtracks; 
+			i++)
 		{
 			if(file->demuxer->vstream_table[i])
 			{
@@ -616,13 +434,11 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 				if(file->vtrack[file->total_vstreams]) 
 					file->total_vstreams++;
 
-//printf("libmpeg3 %d %d %p %d\n", i, file->demuxer->vstream_table[i], file->vtrack[file->total_vstreams], file->total_vstreams);
 			}
 		}
 
-//printf("mpeg3_open 4\n");
 /* Create audio tracks */
-		for(i = 0; i < MPEG3_MAX_STREAMS; i++)
+		for(i = 0; i < MPEG3_MAX_STREAMS && file->total_astreams < toc_atracks; i++)
 		{
 			if(file->demuxer->astream_table[i])
 			{
@@ -634,18 +450,15 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 				if(file->atrack[file->total_astreams]) file->total_astreams++;
 			}
 		}
-//printf("mpeg3_open 5\n");
 	}
 	else
 	if(file->is_video_stream)
 	{
 /* Create video tracks */
-//printf("mpeg3_open 3\n");
 		file->vtrack[0] = mpeg3_new_vtrack(file, 
 			-1, 
 			file->demuxer, 
 			0);
-//printf("mpeg3_open 4\n");
 		if(file->vtrack[0]) file->total_vstreams++;
 	}
 	else
@@ -653,22 +466,20 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 	{
 /* Create audio tracks */
 
-//printf("mpeg3_open 3\n");
 		file->atrack[0] = mpeg3_new_atrack(file, 
 			-1, 
 			AUDIO_UNKNOWN, 
 			file->demuxer,
 			0);
-//printf("mpeg3_open 3\n");
 		if(file->atrack[0]) file->total_astreams++;
 	}
 
-//printf("mpeg3_open 100\n");
+
 
 
 
 	mpeg3io_close_file(file->fs);
-//printf("mpeg3_open 120\n");
+
 	return file;
 }
 
@@ -693,15 +504,6 @@ int mpeg3_set_cpus(mpeg3_t *file, int cpus)
 	return 0;
 }
 
-int mpeg3_set_mmx(mpeg3_t *file, int use_mmx)
-{
-	int i;
-	file->have_mmx = use_mmx;
-	for(i = 0; i < file->total_vstreams; i++)
-		mpeg3video_set_mmx(file->vtrack[i]->video, use_mmx);
-	return 0;
-}
-
 int mpeg3_has_audio(mpeg3_t *file)
 {
 	return file->total_astreams > 0;
@@ -845,34 +647,36 @@ int mpeg3_set_frame(mpeg3_t *file,
 	return -1;
 }
 
-int mpeg3_seek_percentage(mpeg3_t *file, double percentage)
+int mpeg3_seek_byte(mpeg3_t *file, int64_t byte)
 {
 	int i;
 
-	file->percentage_pts = -1;
+//	file->percentage_pts = -1;
 	for(i = 0; i < file->total_vstreams; i++)
 	{
 		file->vtrack[i]->current_position = 0;
-		mpeg3video_seek_percentage(file->vtrack[i]->video, percentage);
+		mpeg3video_seek_byte(file->vtrack[i]->video, byte);
 	}
 
 	for(i = 0; i < file->total_astreams; i++)
 	{
 		file->atrack[i]->current_position = 0;
-		mpeg3audio_seek_percentage(file->atrack[i]->audio, percentage);
+		mpeg3audio_seek_byte(file->atrack[i]->audio, byte);
 	}
 
 	return 0;
 }
 
-double mpeg3_get_percentage_pts(mpeg3_t *file)
-{
-	return file->percentage_pts;
-}
-
-void mpeg3_set_percentage_pts(mpeg3_t *file, double pts)
-{
-}
+/*
+ * double mpeg3_get_percentage_pts(mpeg3_t *file)
+ * {
+ * 	return file->percentage_pts;
+ * }
+ * 
+ * void mpeg3_set_percentage_pts(mpeg3_t *file, double pts)
+ * {
+ * }
+ */
 
 
 int mpeg3_previous_frame(mpeg3_t *file, int stream)
@@ -886,19 +690,24 @@ int mpeg3_previous_frame(mpeg3_t *file, int stream)
 	return 0;
 }
 
-double mpeg3_tell_percentage(mpeg3_t *file)
+int64_t mpeg3_tell_byte(mpeg3_t *file)
 {
-	double percent = 0;
+	int64_t result = 0;
 	if(file->last_type_read == 1)
 	{
-		percent = mpeg3demux_tell_percentage(file->atrack[file->last_stream_read]->demuxer);
+		result = mpeg3demux_tell_byte(file->atrack[file->last_stream_read]->demuxer);
 	}
 
 	if(file->last_type_read == 2)
 	{
-		percent = mpeg3demux_tell_percentage(file->vtrack[file->last_stream_read]->demuxer);
+		result = mpeg3demux_tell_byte(file->vtrack[file->last_stream_read]->demuxer);
 	}
-	return percent;
+	return result;
+}
+
+int64_t mpeg3_get_bytes(mpeg3_t *file)
+{
+	return mpeg3demux_movie_size(file->demuxer);
 }
 
 double mpeg3_get_time(mpeg3_t *file)
@@ -923,14 +732,18 @@ double mpeg3_get_time(mpeg3_t *file)
 /* Use percentage and total time */
 		if(file->total_astreams)
 		{
-			atime = mpeg3demux_tell_percentage(file->atrack[0]->demuxer) * 
-						mpeg3_audio_samples(file, 0) / mpeg3_sample_rate(file, 0);
+			atime = mpeg3demux_tell_byte(file->atrack[0]->demuxer) * 
+						mpeg3_audio_samples(file, 0) / 
+						mpeg3_sample_rate(file, 0) /
+						mpeg3_get_bytes(file);
 		}
 
 		if(file->total_vstreams)
 		{
-			vtime = mpeg3demux_tell_percentage(file->vtrack[0]->demuxer) *
-						mpeg3_video_frames(file, 0) / mpeg3_frame_rate(file, 0);
+			vtime = mpeg3demux_tell_byte(file->vtrack[0]->demuxer) *
+						mpeg3_video_frames(file, 0) / 
+						mpeg3_frame_rate(file, 0) /
+						mpeg3_get_bytes(file);
 		}
 	}
 
@@ -953,40 +766,6 @@ int mpeg3_end_of_video(mpeg3_t *file, int stream)
 }
 
 
-int mpeg3_read_frame(mpeg3_t *file, 
-		unsigned char **output_rows, 
-		int in_x, 
-		int in_y, 
-		int in_w, 
-		int in_h, 
-		int out_w, 
-		int out_h, 
-		int color_model,
-		int stream)
-{
-	int result = -1;
-
-	if(file->total_vstreams)
-	{
-//printf(__FUNCTION__ " 1 %d\n", file->vtrack[stream]->current_position);
-		result = mpeg3video_read_frame(file->vtrack[stream]->video, 
-					file->vtrack[stream]->current_position, 
-					output_rows,
-					in_x, 
-					in_y, 
-					in_w, 
-					in_h, 
-					out_w,
-					out_h,
-					color_model);
-//printf(__FUNCTION__ " 2\n");
-		file->last_type_read = 2;
-		file->last_stream_read = stream;
-		file->vtrack[stream]->current_position++;
-	}
-	return result;
-}
-
 int mpeg3_drop_frames(mpeg3_t *file, long frames, int stream)
 {
 	int result = -1;
@@ -1021,6 +800,42 @@ int mpeg3_set_rowspan(mpeg3_t *file, int bytes, int stream)
 }
 
 
+int mpeg3_read_frame(mpeg3_t *file, 
+		unsigned char **output_rows, 
+		int in_x, 
+		int in_y, 
+		int in_w, 
+		int in_h, 
+		int out_w, 
+		int out_h, 
+		int color_model,
+		int stream)
+{
+	int result = -1;
+//printf("mpeg3_read_frame 1 %d\n", file->vtrack[stream]->current_position);
+
+	if(file->total_vstreams)
+	{
+		result = mpeg3video_read_frame(file->vtrack[stream]->video, 
+					file->vtrack[stream]->current_position, 
+					output_rows,
+					in_x, 
+					in_y, 
+					in_w, 
+					in_h, 
+					out_w,
+					out_h,
+					color_model);
+//printf(__FUNCTION__ " 2\n");
+		file->last_type_read = 2;
+		file->last_stream_read = stream;
+		file->vtrack[stream]->current_position++;
+	}
+
+//printf("mpeg3_read_frame 2 %d\n", file->vtrack[stream]->current_position);
+	return result;
+}
+
 int mpeg3_read_yuvframe(mpeg3_t *file,
 		char *y_output,
 		char *u_output,
@@ -1033,7 +848,7 @@ int mpeg3_read_yuvframe(mpeg3_t *file,
 {
 	int result = -1;
 
-//printf("mpeg3_read_yuvframe 1 %d %d\n", mpeg3demux_tell(file->vtrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->vtrack[stream]->demuxer));
+//printf("mpeg3_read_yuvframe 1\n");
 	if(file->total_vstreams)
 	{
 		result = mpeg3video_read_yuvframe(file->vtrack[stream]->video, 
@@ -1049,7 +864,7 @@ int mpeg3_read_yuvframe(mpeg3_t *file,
 		file->last_stream_read = stream;
 		file->vtrack[stream]->current_position++;
 	}
-//printf("mpeg3_read_yuvframe 2 %d %d\n", mpeg3demux_tell(file->vtrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->vtrack[stream]->demuxer));
+//printf("mpeg3_read_yuvframe 100\n");
 	return result;
 }
 
@@ -1061,7 +876,6 @@ int mpeg3_read_yuvframe_ptr(mpeg3_t *file,
 {
 	int result = -1;
 
-//printf("mpeg3_read_yuvframe_ptr 1 %d %d\n", mpeg3demux_tell(file->vtrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->vtrack[stream]->demuxer));
 	if(file->total_vstreams)
 	{
 		result = mpeg3video_read_yuvframe_ptr(file->vtrack[stream]->video, 
@@ -1073,7 +887,6 @@ int mpeg3_read_yuvframe_ptr(mpeg3_t *file,
 		file->last_stream_read = stream;
 		file->vtrack[stream]->current_position++;
 	}
-//printf("mpeg3_read_yuvframe_ptr 2 %d %d\n", mpeg3demux_tell(file->vtrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->vtrack[stream]->demuxer));
 	return result;
 }
 
@@ -1156,3 +969,6 @@ int mpeg3_read_video_chunk(mpeg3_t *file,
 	}
 	return result;
 }
+
+
+
