@@ -22,31 +22,28 @@ int64_t mpeg3io_get_total_bytes(mpeg3_fs_t *fs);
 /* TITLE */
 
 mpeg3_title_t* mpeg3_new_title(mpeg3_t *file, char *path);
-void mpeg3_new_timecode(mpeg3_title_t *title, 
-		long start_byte, 
-		double start_time,
-		long end_byte,
-		double end_time,
-		int program);
-mpeg3demux_timecode_t* mpeg3_append_timecode(mpeg3_demuxer_t *demuxer, 
-		mpeg3_title_t *title, 
-		long prev_byte, 
-		double prev_time, 
-		long start_byte, 
-		double start_time,
-		int dont_store,
+void mpeg3_new_cell(mpeg3_title_t *title, 
+		int64_t program_start, 
+		int64_t program_end,
+		int64_t title_start,
+		int64_t title_end,
 		int program);
+/* Called by mpeg3_open for a single file */
+int mpeg3demux_create_title(mpeg3_demuxer_t *demuxer, 
+		FILE *toc);
 
 
 /* ATRACK */
 
 mpeg3_atrack_t* mpeg3_new_atrack(mpeg3_t *file, 
-	int stream_id, 
+	int custom_id, 
 	int is_ac3, 
 	mpeg3_demuxer_t *demuxer,
 	int number);
 int mpeg3_delete_atrack(mpeg3_t *file, mpeg3_atrack_t *atrack);
 
+void mpeg3_append_samples(mpeg3_atrack_t *atrack, int64_t offset);
+
 
 /* These return 1 on failure and 0 on success */
 int mpeg3_next_header();
@@ -54,16 +51,31 @@ int mpeg3_next_header();
 /* VTRACK */
 
 mpeg3_vtrack_t* mpeg3_new_vtrack(mpeg3_t *file, 
-	int stream_id, 
+	int custom_id, 
 	mpeg3_demuxer_t *demuxer,
 	int number);
 int mpeg3_delete_vtrack(mpeg3_t *file, mpeg3_vtrack_t *vtrack);
 
+void mpeg3_append_frame(mpeg3_vtrack_t *vtrack, int64_t offset, int is_keyframe);
+
+
 /* AUDIO */
-mpeg3audio_t* mpeg3audio_new(mpeg3_t *file, mpeg3_atrack_t *track, int is_ac3);
+mpeg3audio_t* mpeg3audio_new(mpeg3_t *file, 
+	mpeg3_atrack_t *track, 
+	int is_ac3);
 int mpeg3audio_delete(mpeg3audio_t *audio);
-/* Read header and store common parameters in audio structure */
-int mpeg3audio_read_header(mpeg3audio_t *audio);
+
+/* Decode up to requested number of samples. */
+/* Return 0 on success and 1 on failure. */
+int mpeg3audio_decode_audio(mpeg3audio_t *audio, 
+	float *output_f, 
+	short *output_i, 
+	int channel,
+	int len);
+
+/* Shift the audio by the number of samples */
+/* Used by table of contents routines and decode_audio */
+void mpeg3_shift_audio(mpeg3audio_t *audio, int diff);
 
 
 /* Audio consists of many possible formats, each packetized into frames. */
@@ -153,11 +165,22 @@ int mpeg3audio_dopcm(mpeg3_pcm_t *audio,
 	int render);
 
 
+
+
+
+
+
+
+
+
+
+
 /* VIDEO */
-mpeg3video_t* mpeg3video_new(mpeg3_t *file, mpeg3_vtrack_t *track);
+mpeg3video_t* mpeg3video_new(mpeg3_t *file, 
+	mpeg3_vtrack_t *track);
 int mpeg3video_delete(mpeg3video_t *video);
 int mpeg3video_read_frame(mpeg3video_t *video, 
-		long frame_number, 
+		int frame_number, 
 		unsigned char **output_rows,
 		int in_x, 
 		int in_y, 
@@ -167,6 +190,10 @@ int mpeg3video_read_frame(mpeg3video_t *video,
 		int out_h, 
 		int color_model);
 void mpeg3video_dump(mpeg3video_t *video);
+int mpeg3video_prev_code(mpeg3_demuxer_t *demuxer, unsigned int code);
+int mpeg3video_next_code(mpeg3_bits_t* stream, unsigned int code);
+void mpeg3video_toc_error();
+int mpeg3_rewind_video(mpeg3video_t *video);
 
 
 
@@ -177,9 +204,25 @@ int mpeg3_delete_fs(mpeg3_fs_t *fs);
 int mpeg3io_open_file(mpeg3_fs_t *fs);
 int mpeg3io_close_file(mpeg3_fs_t *fs);
 int mpeg3io_seek(mpeg3_fs_t *fs, int64_t byte);
-int mpeg3io_read_data(unsigned char *buffer, long bytes, mpeg3_fs_t *fs);
-int mpeg3io_next_code(mpeg3_fs_t *fs, uint32_t code, int count);
-int mpeg3io_prev_code(mpeg3_fs_t *fs, uint32_t code, int count);
+int mpeg3io_seek_relative(mpeg3_fs_t *fs, int64_t bytes);
+int mpeg3io_read_data(unsigned char *buffer, int64_t bytes, mpeg3_fs_t *fs);
+
+
+
+
+
+
+/* MAIN */
+mpeg3_t* mpeg3_new(char *path);
+mpeg3_index_t* mpeg3_new_index();
+void mpeg3_delete_index(mpeg3_index_t *index);
+int mpeg3_delete(mpeg3_t *file);
+int mpeg3_get_file_type(mpeg3_t *file, 
+	mpeg3_t *old_file,
+	int *toc_atracks,
+	int *toc_vtracks);
+
+int mpeg3_read_toc(mpeg3_t *file, int *atracks_return, int *vtracks_return);
 
 
 
@@ -191,41 +234,54 @@ int mpeg3io_prev_code(mpeg3_fs_t *fs, uint32_t code, int count);
 
 
 
-
-
-
-
-// Demuxer
+/* DEMUXER */
 
 
 
 mpeg3_demuxer_t* mpeg3_new_demuxer(mpeg3_t *file, 
 	int do_audio, 
 	int do_video, 
-	int stream_id);
+	int custom_id);
 int mpeg3_delete_demuxer(mpeg3_demuxer_t *demuxer);
+mpeg3_demuxer_t* mpeg3_get_demuxer(mpeg3_t *file);
 int mpeg3demux_read_data(mpeg3_demuxer_t *demuxer, 
 	unsigned char *output, 
-	long size);
+	int size);
+
+/* Append elementary stream data */
+/* Used by streaming mode. */
+void mpeg3demux_append_data(mpeg3_demuxer_t *demuxer, 
+	unsigned char *data, 
+/* Bytes of data */
+	int bytes);
+
+/* Shift elementary data out */
+/* Used by streaming mode */
+void mpeg3demux_shift_data(mpeg3_demuxer_t *demuxer,
+	int bytes);
+
+/* Convert absolute byte position to position in program */
+int64_t mpeg3_absolute_to_program(mpeg3_demuxer_t *demuxer,
+	int64_t absolute_byte);
+
+
+int mpeg3_read_next_packet(mpeg3_demuxer_t *demuxer);
+int mpeg3_read_prev_packet(mpeg3_demuxer_t *demuxer);
+
+
 unsigned int mpeg3demux_read_int32(mpeg3_demuxer_t *demuxer);
 unsigned int mpeg3demux_read_int24(mpeg3_demuxer_t *demuxer);
 unsigned int mpeg3demux_read_int16(mpeg3_demuxer_t *demuxer);
-double mpeg3demux_length(mpeg3_demuxer_t *demuxer);
-mpeg3_demuxer_t* mpeg3_get_demuxer(mpeg3_t *file);
 
-/* Give byte offset relative to current title */
-int64_t mpeg3demux_tell(mpeg3_demuxer_t *demuxer);
 
-/* Give byte offset relative to start of movie */
-int64_t mpeg3demux_tell_absolute(mpeg3_demuxer_t *demuxer);
-
-/* Give total number of bytes in all titles */
+/* Give total number of bytes in all titles which belong to the current program. */
 int64_t mpeg3demux_movie_size(mpeg3_demuxer_t *demuxer);
 
-double mpeg3demux_tell_percentage(mpeg3_demuxer_t *demuxer);
+/* Give byte offset relative to start of movie */
+int64_t mpeg3demux_tell_byte(mpeg3_demuxer_t *demuxer);
 
-/* Return the current title */
-int mpeg3demux_tell_title(mpeg3_demuxer_t *demuxer);
+/* Give program the current packet belongs to */
+int mpeg3demux_tell_program(mpeg3_demuxer_t *demuxer);
 
 double mpeg3demux_get_time(mpeg3_demuxer_t *demuxer);
 int mpeg3demux_eof(mpeg3_demuxer_t *demuxer);
@@ -233,9 +289,14 @@ int mpeg3demux_bof(mpeg3_demuxer_t *demuxer);
 void mpeg3demux_start_reverse(mpeg3_demuxer_t *demuxer);
 void mpeg3demux_start_forward(mpeg3_demuxer_t *demuxer);
 int mpeg3demux_open_title(mpeg3_demuxer_t *demuxer, int title_number);
+/* Go to the absolute byte given */
 int mpeg3demux_seek_byte(mpeg3_demuxer_t *demuxer, int64_t byte);
-int mpeg3demux_seek_percentage(mpeg3_demuxer_t *demuxer, double percentage);
-int64_t mpeg3demuxer_total_bytes(mpeg3_demuxer_t *demuxer);
+
+/* Seek to the title and cell containing the absolute byte of the 
+/* demuxer. */
+/* Called at the beginning of every packet. */
+int mpeg3_seek_phys(mpeg3_demuxer_t *demuxer);
+
 unsigned char mpeg3demux_read_char_packet(mpeg3_demuxer_t *demuxer);
 unsigned char mpeg3demux_read_prev_char_packet(mpeg3_demuxer_t *demuxer);
 int mpeg3demux_read_program(mpeg3_demuxer_t *demuxer);
@@ -248,10 +309,10 @@ double mpeg3demux_video_pts(mpeg3_demuxer_t *demuxer);
 /* Set the last pts read to -1 for audio and video */
 void mpeg3demux_reset_pts(mpeg3_demuxer_t *demuxer);
 
-/* scan forward for next pts.  Used in percentage seeking to synchronize */
+/* scan forward for next pts.  Used in byte seeking to synchronize */
 double mpeg3demux_scan_pts(mpeg3_demuxer_t *demuxer);
 
-/* seek using sequential search to the pts given.  Used in percentage seeking. */
+/* seek using sequential search to the pts given.  Used in byte seeking. */
 int mpeg3demux_goto_pts(mpeg3_demuxer_t *demuxer, double pts);
 
 
@@ -260,10 +321,6 @@ unsigned char mpeg3demux_read_prev_char_packet(mpeg3_demuxer_t *demuxer);
 
 #define mpeg3demux_error(demuxer) (((mpeg3_demuxer_t *)(demuxer))->error_flag)
 
-#define mpeg3demux_time_offset(demuxer) (((mpeg3_demuxer_t *)(demuxer))->time_offset)
-
-#define mpeg3demux_current_time(demuxer) (((mpeg3_demuxer_t *)(demuxer))->time + ((mpeg3_demuxer_t *)(demuxer))->time_offset)
-
 static unsigned char mpeg3demux_read_char(mpeg3_demuxer_t *demuxer)
 {
 //printf("mpeg3demux_read_char %lx %lx\n", demuxer->data_position, demuxer->data_size);
@@ -309,19 +366,16 @@ mpeg3_bits_t* mpeg3bits_new_stream(mpeg3_t *file, mpeg3_demuxer_t *demuxer);
 int mpeg3bits_delete_stream(mpeg3_bits_t* stream);
 int mpeg3bits_seek_byte(mpeg3_bits_t* stream, int64_t position);
 int mpeg3bits_open_title(mpeg3_bits_t* stream, int title);
+/* Give absolute byte offset in all titles. */
 int64_t mpeg3bits_tell(mpeg3_bits_t* stream);
+/* Reset bit bucket */
+void mpeg3bits_reset(mpeg3_bits_t *stream);
 
 
 
 
 
 
-#define mpeg3bits_tell_percentage(stream) mpeg3demux_tell_percentage((stream)->demuxer)
-
-#define mpeg3bits_packet_time(stream) mpeg3demux_current_time((stream)->demuxer)
-
-#define mpeg3bits_time_offset(stream) mepg2demux_time_offset((stream)->demuxer)
-
 #define mpeg3bits_error(stream) mpeg3demux_error((stream)->demuxer)
 
 #define mpeg3bits_eof(stream) mpeg3demux_eof((stream)->demuxer)
@@ -473,7 +527,7 @@ static unsigned int mpeg3bits_showbits_reverse(mpeg3_bits_t* stream, int bits)
 
 
 // I/O
-
+// I/O must be character based so the buffer doesn't get overrun
 
 
 
@@ -497,8 +551,8 @@ void mpeg3io_read_buffer(mpeg3_fs_t *fs);
 
 static int mpeg3io_sync_buffer(mpeg3_fs_t *fs)
 {
-	if(fs->buffer_position + fs->buffer_offset != fs->current_byte)
 // Reposition buffer offset
+	if(fs->buffer_position + fs->buffer_offset != fs->current_byte)
 	{
 		fs->buffer_offset = fs->current_byte - fs->buffer_position;
 	}
@@ -509,7 +563,7 @@ static int mpeg3io_sync_buffer(mpeg3_fs_t *fs)
 	{
 		mpeg3io_read_buffer(fs);
 	}
-	
+
 	return !fs->buffer_size;
 }
 
